@@ -4,15 +4,15 @@ import logging
 import threading
 import sys
 import traceback
-from typing import Iterable, TYPE_CHECKING, Union
+from typing import Iterable, Union
 from collections import deque
 from enum import IntEnum, auto
 import time
+import asyncio
 
 from .stream_packets import FeedbackEvent, FeedbackHeader, ControllerState, StickState
-
-if TYPE_CHECKING:
-    from .session import Session
+from .errors import RemotePlayError
+from .session import Session
 
 _LOGGER = logging.getLogger(__name__)
 
@@ -36,30 +36,41 @@ class Controller:
         """Return list of valid buttons."""
         return [button.name for button in FeedbackEvent.Type]
 
-    def __init__(self, session=None, **kwargs):
+    def __init__(self, session=None):
         self._session = session
         self._sequence_event = 0
         self._sequence_state = 0
         self._event_buf = deque([], Controller.MAX_EVENTS)
-        self._buttons = {}
-        self._params = kwargs
-        self._started = False
-        self._should_send = threading.Semaphore()
-        self._stop_event = threading.Event()
-        self._thread: threading.Thread = None
         self._last_state = ControllerState()
         self._stick_state = ControllerState()
 
+        self._should_send = threading.Semaphore()
+        self._stop_event = threading.Event()
+        self._thread: threading.Thread = None
+
     def __del__(self):
         self.disconnect()
 
+    def __reset_session(self):
+        self._sequence_event = 0
+        self._sequence_state = 0
+        self._event_buf = deque([], Controller.MAX_EVENTS)
+        self._last_state = ControllerState()
+        self._stick_state = ControllerState()
+
+    def __reset_worker(self):
+        self._should_send = threading.Semaphore()
+        self._stop_event = threading.Event()
+        self._thread = None
+
     def __worker(self):
         """Worker for sending feedback packets. Run in thread."""
         self._should_send.acquire(timeout=1)
         while self.running:
             try:
                 self._should_send.acquire(timeout=Controller.STATE_INTERVAL_MAX_MS)
-                self.update_sticks()
+                if self.ready:
+                    self.update_sticks()
             except Exception as error:  # pylint: disable=broad-except
                 _LOGGER.error("Error in controller thread: %s", error)
                 if _LOGGER.level == logging.DEBUG:
@@ -67,34 +78,40 @@ class Controller:
                     traceback.print_exception(
                         exc_type, exc_value, exc_traceback, file=sys.stdout
                     )
-        self._session = None
-        self._thread = None
-        self._stop_event.clear()
-        self._should_send = threading.Semaphore()
+        self.__reset_worker()
         _LOGGER.info("Controller stopped")
 
     def connect(self, session: Session):
         """Connect controller to session."""
-        if not isinstance(session, Session):
-            raise TypeError(f"Expected instance of {Session}")
         if self._session is not None:
-            _LOGGER.warning("Controller already connected. Call disconnect first")
+            _LOGGER.warning("Controller already connected. Call `disconnect()` first")
             return
+
+        if session is not None:
+            if not isinstance(session, Session):
+                raise TypeError(f"Expected {Session}. Got {type(session)}")
+            if session.is_running:
+                raise RemotePlayError("Cannot set a running session")
+            if session.is_stopped:
+                raise RemotePlayError("Cannot set a stopped session")
+
+        self.__reset_session()
+        self.__reset_worker()
         self._session = session
 
     def start(self):
         """Start Controller.
 
-        This starts the controller worker which listens for when the sticks changes and sends the state to the host.
-        If this is not called, the
+        This starts the controller worker which listens for when the sticks move
+        and sends the state to the host. If this is not called, the
         :meth:`update_sticks() <pyremoteplay.controller.Controller.update_sticks>`
         method needs to be called for the host to receive the state.
         """
         if self._thread is not None:
-            _LOGGER.warning("Controller is running. Call stop first")
+            _LOGGER.warning("Controller is running. Call `stop()` first")
             return
         if self._session is None:
-            _LOGGER.warning("Controller has no session. Call connect first")
+            _LOGGER.warning("Controller has no session. Call `connect()` first")
             return
         self._thread = threading.Thread(target=self.__worker, daemon=True)
         self._thread.start()
@@ -104,8 +121,9 @@ class Controller:
         self._stop_event.set()
 
     def disconnect(self):
-        """Stop and Disconnect Controller."""
+        """Stop and Disconnect Controller. Must be called to change session."""
         self.stop()
+        self.__reset_session()
         self._session = None
 
     def update_sticks(self):
@@ -114,11 +132,9 @@ class Controller:
         Will be called automatically if controller has been started with
         :meth:`start() <pyremoteplay.controller.Controller.start>`.
         """
-        if (
-            self.session is None
-            or self.session.is_stopped
-            or self.stick_state == self._last_state
-        ):
+        if not self._check_session():
+            return
+        if self.stick_state == self._last_state:
             return
         self._last_state.left = self.stick_state.left
         self._last_state.right = self.stick_state.right
@@ -129,13 +145,11 @@ class Controller:
 
     def _send_event(self):
         """Send controller button event."""
-        if self.session is None or self.session.is_stopped:
-            return
         data = b"".join(self._event_buf)
         if not data:
             return
         self._session.stream.send_feedback(
-            FeedbackHeader.Type.EVENT, self.sequence_event, data=data
+            FeedbackHeader.Type.EVENT, self._sequence_event, data=data
         )
         self._sequence_event += 1
 
@@ -148,6 +162,39 @@ class Controller:
         event.pack(buf)
         self._event_buf.appendleft(buf)
 
+    def _button(
+        self,
+        name: Union[str, FeedbackEvent.Type],
+        action: Union[str, ButtonAction],
+    ) -> tuple[FeedbackEvent.Type, ButtonAction]:
+        if not self._check_session():
+            return None
+        if isinstance(action, self.ButtonAction):
+            _action = action
+        else:
+            try:
+                _action = self.ButtonAction[action.upper()]
+            except KeyError:
+                _LOGGER.error("Invalid Action: %s", action)
+                return None
+        if isinstance(name, FeedbackEvent.Type):
+            button = name
+        else:
+            try:
+                button = FeedbackEvent.Type[name.upper()]
+            except KeyError:
+                _LOGGER.error("Invalid button: %s", name)
+                return None
+
+        if _action == self.ButtonAction.PRESS:
+            self._add_event_buffer(FeedbackEvent(button, is_active=True))
+        elif _action == self.ButtonAction.RELEASE:
+            self._add_event_buffer(FeedbackEvent(button, is_active=False))
+        elif _action == self.ButtonAction.TAP:
+            self._add_event_buffer(FeedbackEvent(button, is_active=True))
+        self._send_event()
+        return button, _action
+
     def button(
         self,
         name: Union[str, FeedbackEvent.Type],
@@ -162,34 +209,35 @@ class Controller:
         :param action: One of `press`, `release`, `tap`, or `Controller.ButtonAction`.
         :param delay: Delay between press and release. Only used when action is `tap`.
         """
-        if isinstance(action, self.ButtonAction):
-            _action = action
-        else:
-            try:
-                _action = self.ButtonAction[action.upper()]
-            except KeyError:
-                _LOGGER.error("Invalid Action: %s", action)
-                return
-        if isinstance(name, FeedbackEvent.Type):
-            button = int(name)
-        else:
-            try:
-                button = int(FeedbackEvent.Type[name.upper()])
-            except KeyError:
-                _LOGGER.error("Invalid button: %s", name)
-                return
-
-        if _action == self.ButtonAction.PRESS:
-            self._add_event_buffer(FeedbackEvent(button, is_active=True))
-        elif _action == self.ButtonAction.RELEASE:
-            self._add_event_buffer(FeedbackEvent(button, is_active=False))
-        elif _action == self.ButtonAction.TAP:
-            self._add_event_buffer(FeedbackEvent(button, is_active=True))
-            self._send_event()
-            time.sleep(delay)
-            self.button(name, self.ButtonAction.RELEASE)
+        data = self._button(name, action)
+        if not data:
             return
-        self._send_event()
+        button, _action = data
+        if _action == self.ButtonAction.TAP:
+            time.sleep(delay)
+            self.button(button, self.ButtonAction.RELEASE)
+
+    async def async_button(
+        self,
+        name: Union[str, FeedbackEvent.Type],
+        action: Union[str, ButtonAction] = "tap",
+        delay=0.1,
+    ):
+        """Emulate pressing or releasing button. Async.
+
+        If action is `tap` this coroutine will sleep by delay.
+
+        :param name: The name of button. Use buttons() to show valid buttons.
+        :param action: One of `press`, `release`, `tap`, or `Controller.ButtonAction`.
+        :param delay: Delay between press and release. Only used when action is `tap`.
+        """
+        data = self._button(name, action)
+        if not data:
+            return
+        button, _action = data
+        if _action == self.ButtonAction.TAP:
+            await asyncio.sleep(delay)
+            await self.async_button(button, self.ButtonAction.RELEASE)
 
     def stick(
         self,
@@ -256,10 +304,17 @@ class Controller:
             self._stick_state.right = state
         self._should_send.release()
 
-    @property
-    def sequence_event(self) -> int:
-        """Return Sequence Number for events."""
-        return self._sequence_event
+    def _check_session(self) -> bool:
+        if self.session is None:
+            _LOGGER.warning("Controller has no session")
+            return False
+        if self.session.is_stopped:
+            _LOGGER.warning("Session is stopped")
+            return False
+        if not self.session.is_ready:
+            _LOGGER.warning("Session is not ready")
+            return False
+        return True
 
     @property
     def stick_state(self) -> ControllerState:
@@ -273,6 +328,13 @@ class Controller:
             return False
         return not self._session.is_stopped and not self._stop_event.is_set()
 
+    @property
+    def ready(self) -> bool:
+        """Return True if controller can be used"""
+        if not self.session:
+            return False
+        return self.session.is_ready
+
     @property
     def session(self) -> Session:
         """Return Session."""
