@@ -327,8 +327,7 @@ class RPDevice:
             if not await self.session.async_wait():
                 _LOGGER.error("Timed out waiting for stream to start")
                 return False
-        self.session.standby()  # TODO: Acually determine if this was successful
-        return True
+        return await self.session.async_standby()
 
     def wakeup(
         self,
